import 'dart:io';

import 'package:flutter/material.dart';

import 'package:vivanta_connect_flutter/data/brand.dart';
import 'package:vivanta_connect_flutter/data/client.dart';
import 'package:vivanta_connect_flutter/helpers/helpers.dart';
import 'package:vivanta_connect_flutter/helpers/vivanta_sync.dart';
import 'package:vivanta_connect_flutter/services/apple_health.dart';
import 'package:vivanta_connect_flutter/services/database.dart';
import 'package:vivanta_connect_flutter/services/google_health_connect.dart';
import 'package:vivanta_connect_flutter/services/http.dart';
import 'package:vivanta_connect_flutter/services/preferences.dart';
import 'package:vivanta_connect_flutter/styles/colors.dart';
import 'package:vivanta_connect_flutter/styles/fonts.dart';
import 'package:vivanta_connect_flutter/views/loading.dart';
import 'package:vivanta_connect_flutter/views/webview.dart';
import 'package:vivanta_connect_flutter/widgets/brand_item.dart';

const appleHealthId = 6;
const googleHealthConnectId = 8;

class Brands extends StatefulWidget {
  const Brands({
    Key? key,
    this.client,
    this.onBrandConnected,
  }) : super(key: key);

  final Client? client;
  final Function(int)? onBrandConnected;

  @override
  State<Brands> createState() => _BrandsState();
}

class _BrandsState extends State<Brands> {
  bool _loading = false;
  bool showSearch = false;
  String _apiKey = '';
  String _customerId = '';
  String _externalUserId = '';
  String _token = '';
  List<Brand> _webBrands = [];
  List<BrandItem> _filteredBrands = [];
  List<HealthPermission> _appleHealthPermissions = [];
  List<HealthPermission> _googleHealthPermissions = [];
  TextEditingController searchController = TextEditingController();
  FocusNode searchFocus = FocusNode();
  final http = Http();
  final storage = Preferences();

  @override
  void initState() {
    _getBrands();
    searchController.addListener(() => setState(() {}));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return _loading
        ? const Loading()
        : Scaffold(
            backgroundColor: VivantaColors.background,
            appBar: AppBar(
              backgroundColor: VivantaColors.background,
              elevation: 0,
              title: const Center(
                child: Image(
                  image: AssetImage(
                    "images/vivanta-logo.png",
                    package: 'vivanta_connect_flutter',
                  ),
                  height: 20,
                  fit: BoxFit.cover,
                ),
              ),
              actions: [
                Padding(
                  padding: const EdgeInsets.only(right: 16),
                  child: GestureDetector(
                    onTap: () {
                      Navigator.of(context).pushReplacementNamed('/close');
                    },
                    child: const Icon(
                      Icons.close,
                      color: VivantaColors.text,
                      size: 24,
                    ),
                  ),
                ),
              ],
              leading: GestureDetector(
                onTap: () {
                  Navigator.of(context).maybePop();
                },
                child: const Icon(
                  Icons.arrow_back_ios,
                  color: VivantaColors.text,
                  size: 24,
                ),
              ),
            ),
            body: Column(
              children: [
                Padding(
                  padding:
                      const EdgeInsets.symmetric(horizontal: 32, vertical: 8),
                  child: TextFormField(
                    decoration: InputDecoration(
                      hintText: "Encuentra una marca",
                      hintStyle: VivantaFonts.title.copyWith(height: 1),
                      labelStyle: VivantaFonts.title.copyWith(height: 1),
                      contentPadding: const EdgeInsets.all(4),
                      isDense: true,
                    ),
                    style: VivantaFonts.title,
                    controller: searchController,
                    focusNode: searchFocus,
                  ),
                ),
                Expanded(
                  child: GridView.count(
                    crossAxisCount: 2,
                    children: _filteredBrands,
                  ),
                ),
              ],
            ));
  }

  Future<void> _getBrands() async {
    setState(() {
      _loading = true;
    });
    _apiKey = await storage.getApiKey();
    _customerId = await storage.getCustomerId();
    _externalUserId = await storage.getExternalUserId();
    _token = await storage.getToken();
    _webBrands = await http.getBrands(_token);
    await _getBrandItems();
    setState(() {
      _loading = false;
    });
  }

  Future<void> _requestAppleHealthPermission() async {
    final database = Database();
    final preferences = Preferences();
    await database.init();
    final appleHealth = AppleHealth();
    final authorized =
        await appleHealth.requestAuthorization(_appleHealthPermissions);

    if (authorized) {
      await preferences.setAppleHealthAccessGranted(authorized);
      await preferences.setAppleHealthAccessGrantedDate(DateTime.now());

      if (context.mounted) {
        await showDialog(
          context: context,
          builder: (context) => AlertDialog(
            title: const Text("Conexión exitosa"),
            content: const Text(
              "Puedes controlar el acceso a los datos de salud desde las opciones del teléfono.",
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context),
                child: const Text("Aceptar"),
              ),
            ],
          ),
        );
      }

      if (widget.onBrandConnected != null) {
        widget.onBrandConnected!(6);
      }

      final vivantaSync = _apiKey.isEmpty
          ? VivantaSync(
              apiKey: '',
              customerId: '',
              externalUserId: '',
              token: _token,
            )
          : VivantaSync(
              apiKey: _apiKey,
              customerId: _customerId,
              externalUserId: _externalUserId,
            );

      vivantaSync.executeAll();

      return;
    } else {
      if (context.mounted) {
        await showDialog(
          context: context,
          builder: (context) => AlertDialog(
            title: const Text("Error al conectar Apple Health"),
            content: const Text(
              "Por favor verifica en las opciones del teléfono que esta aplicación puede acceder a Apple Health.",
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context),
                child: const Text("Aceptar"),
              ),
            ],
          ),
        );
      }
      return;
    }
  }

  Future<void> _requestGoogleHealthConnectPermission() async {
    final database = Database();
    final preferences = Preferences();
    await database.init();
    final healthConnect = GoogleHealthConnect();

    final isHealthConnectInstalled = await healthConnect.isInstalled();

    if (!isHealthConnectInstalled) {
      if (context.mounted) {
        final dialogResult = await showDialog<bool>(
          context: context,
          builder: (context) => AlertDialog(
            title: const Text("Health Connect no instalado"),
            content: const Text(
              "¿Deseas que te dirijamos a la Play Store para instalar la aplicación?",
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                child: const Text("Sí"),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                child: const Text("No"),
              ),
            ],
          ),
        );

        if (dialogResult != null) {
          if (dialogResult) {
            healthConnect.install();
          }
        }
      }
      return;
    }

    final isGoogleFitInstalled = await healthConnect.isGoogleFitInstalled();

    if (!isGoogleFitInstalled) {
      if (context.mounted) {
        final dialogResult = await showDialog<bool>(
          context: context,
          builder: (context) => AlertDialog(
            title: const Text("Google Fit no instalado"),
            content: const Text(
              "Google Fit complementa la experiencia de Health Connect. ¿Deseas que te dirijamos a la Play Store para instalar la aplicación?",
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                child: const Text("Sí"),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                child: const Text("No"),
              ),
            ],
          ),
        );

        if (dialogResult != null) {
          if (dialogResult) {
            healthConnect.installGoogleFit();
          }
        }
      }

      return;
    }

    final granted = await preferences.getGoogleHealthConnectAccessGranted();
    final authorized =
        await healthConnect.requestAuthorization(_googleHealthPermissions);

    if (authorized) {
      if (!granted) {
        await preferences.setGoogleHealthConnectAccessGranted(authorized);
        await preferences
            .setGoogleHealthConnectAccessGrantedDate(DateTime.now());
      }
      if (context.mounted) {
        final dialogResult = await showDialog<bool>(
          context: context,
          builder: (context) => AlertDialog(
            title: const Text("Conexión exitosa"),
            content: const Text(
              "¿Deseas ir a la configuración de Health Connect?",
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                child: const Text("Sí"),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                child: const Text("No"),
              ),
            ],
          ),
        );

        if (dialogResult != null) {
          if (dialogResult) {
            healthConnect.open();
          } else {
            if (widget.onBrandConnected != null) {
              widget.onBrandConnected!(8);
            }
            final vivantaSync = _apiKey.isEmpty
                ? VivantaSync(
                    apiKey: '',
                    customerId: '',
                    externalUserId: '',
                    token: _token,
                  )
                : VivantaSync(
                    apiKey: _apiKey,
                    customerId: _customerId,
                    externalUserId: _externalUserId,
                  );

            vivantaSync.executeAll();
          }
        }
      }

      return;
    }

    if (context.mounted) {
      final dialogResult = await showDialog<bool>(
        context: context,
        builder: (context) => AlertDialog(
          title: const Text("Error al conectar Health Connect"),
          content:
              const Text("¿Deseas ir a la configuración de Health Connect?"),
          actions: [
            TextButton(
              onPressed: () => Navigator.pop(context, true),
              child: const Text("Sí"),
            ),
            TextButton(
              onPressed: () => Navigator.pop(context, false),
              child: const Text("No"),
            ),
          ],
        ),
      );

      if (dialogResult != null) {
        if (dialogResult) {
          healthConnect.open();
        }
      }
    }

    return;
  }

  Future<void> _getBrandItems() async {
    _appleHealthPermissions =
        await getAppleHealthPermissionFromClient(widget.client);
    _googleHealthPermissions =
        await getGoogleHealthPermissionFromClient(widget.client);

    final hasAppleHealth = _webBrands.any((element) => element.id == 6);
    final hasGoogleHealth = _webBrands.any((element) => element.id == 8);

    if (!Platform.isIOS) {
      _removeAppleHealth();
    } else if (_appleHealthPermissions.isEmpty) {
      _removeAppleHealth();
    }

    if (!Platform.isAndroid) {
      _removeGoogleHealthConnect();
    } else if (_googleHealthPermissions.isEmpty) {
      _removeGoogleHealthConnect();
    }

    if (!hasAppleHealth &&
        _appleHealthPermissions.isNotEmpty &&
        Platform.isIOS) {
      _addAppleHealth();
    }

    if (!hasGoogleHealth &&
        _googleHealthPermissions.isNotEmpty &&
        Platform.isAndroid) {
      _addGoogleHealthConnect();
    }

    final filteredBrands = _webBrands
        .where((element) => element.name!
            .toLowerCase()
            .contains(searchController.text.toLowerCase()))
        .toList();

    filteredBrands.sort(
      (a, b) => a.name!.toLowerCase().compareTo(b.name!.toLowerCase()),
    );

    _filteredBrands = filteredBrands
        .map<BrandItem>(
          (brand) => BrandItem(
            brand: brand,
            onTap: (brand) {
              if (brand.id == 6) {
                _requestAppleHealthPermission();
                return;
              }
              if (brand.id == 8) {
                _requestGoogleHealthConnectPermission();
                return;
              }
              final url = _apiKey.isEmpty
                  ? 'https://connect.vivanta.io/native-with-token/$_token/${brand.id}'
                  : 'https://connect.vivanta.io/native/$_apiKey/$_customerId/$_externalUserId/${brand.id}';
              Navigator.of(context).push(
                MaterialPageRoute(
                  builder: (context) => Webview(
                    brand: brand,
                    url: url,
                  ),
                ),
              );
            },
          ),
        )
        .toList();

    setState(() {
      _filteredBrands = _filteredBrands;
    });
  }

  void _removeAppleHealth() {
    _webBrands.removeWhere((element) => element.id == appleHealthId);
  }

  void _addAppleHealth() {
    final apple = Brand(
      id: appleHealthId,
      name: 'Apple',
      image: 'https://vivanta-doc.s3.amazonaws.com/brand-apple-square.png',
    );
    _webBrands.add(apple);
  }

  void _removeGoogleHealthConnect() {
    _webBrands.removeWhere((element) => element.id == googleHealthConnectId);
  }

  void _addGoogleHealthConnect() {
    final google = Brand(
      id: googleHealthConnectId,
      name: 'Google Health Connect',
      image:
          'https://vivanta-doc.s3.amazonaws.com/brand-google-health-square.png',
    );
    _webBrands.add(google);
  }

  @override
  void dispose() {
    searchController.dispose();
    searchFocus.dispose();
    super.dispose();
  }
}
